# High level library to do opc-xmlda operations

from OPCServer_client import *


def write (port, item_name, value):
    msg = WriteSoapIn ()

    options = msg.new_Options ()

    msg.set_element_Options (options)

    item_list = msg.new_ItemList ()

    items = item_list.new_Items ()

    items.set_attribute_ItemName (item_name)

    items.Value = value

    item_list.set_element_Items ([items,])

    msg.set_element_ItemList (item_list)

    rsp = port.Write (msg)


def read (port, item_name, callback):
    msg = ReadSoapIn ()

    options = msg.new_Options ()

    options.set_attribute_ReturnItemName (True)
    options.set_attribute_ReturnItemTime (True)

    msg.set_element_Options (options)

    item_list = msg.new_ItemList ()

    items = item_list.new_Items ()

    items.set_attribute_ItemName (item_name)

    item_list.set_element_Items ([items,])

    msg.set_element_ItemList (item_list)

    rsp = port.Read (msg)

    item_list = rsp.get_element_RItemList ()

    for i in item_list.get_element_Items ():
        callback (i)


def browse (port, root, indent, element_callback, item_callback):
    msg = BrowseSoapIn ()
    msg.set_attribute_ItemName (root)

    rsp = port.Browse (msg)

    for i in rsp.Elements:
        element_callback (i, indent)

        if i.get_attribute_HasChildren ():
            browse (port, 
                    i.get_attribute_ItemName (), 
                    indent + 1, 
                    element_callback,
                    item_callback)

        if i.get_attribute_IsItem ():
            read (port, i.get_attribute_ItemName (), 
                        (lambda x: item_callback (x, indent + 1)))


def subscribe (port, item_names, ping_rate=4000):

    msg = SubscribeSoapIn ()

    msg.set_attribute_SubscriptionPingRate (ping_rate)
    msg.set_attribute_ReturnValuesOnReply (False)

    options = msg.new_Options ()

    options.set_attribute_ReturnItemName (True)
    options.set_attribute_ReturnItemTime (True)

    msg.set_element_Options (options)

    item_list = msg.new_ItemList ()
    list_of_items = [ ]

    for i in item_names:
        item = item_list.new_Items ()
        item.set_attribute_ItemName (i)
        list_of_items.append (item)

    item_list.set_element_Items (list_of_items)
    msg.set_element_ItemList (item_list)

    rsp = port.Subscribe (msg)

    return rsp.get_attribute_ServerSubHandle ()


def cancel_subscription (port, handle):
    msg = SubscriptionCancelSoapIn ()

    msg.set_attribute_ServerSubHandle (handle)

    rsp = port.SubscriptionCancel (msg)


def poll (port, handle, iterations, callback):

    for i in xrange (iterations):    
        msg = SubscriptionPolledRefreshSoapIn ()

        msg.set_attribute_WaitTime (5000)
#        msg.set_attribute_HoldTime ((2009, 5, 5, 16, 24, 9, 3, 194, 0))

        options = msg.new_Options ()

        options.set_attribute_ReturnItemName (True)
        options.set_attribute_ReturnItemTime (True)
        options.set_attribute_ReturnItemPath (True)

        msg.set_element_Options (options)
        msg.set_element_ServerSubHandles ([handle,])

        rsp = port.SubscriptionPolledRefresh (msg)

        for i in rsp.get_element_RItemList ():
            for j in i.get_element_Items ():
                callback (j)
